import type { Member } from '$lib/model/Member';
import jsPDF from 'jspdf';
import autoTable from 'jspdf-autotable';
import { printHeader, printPlaceDate } from './common';

const emptySlotCount = 5;

export function generateSecretVotingList({
	votingItems,
	members
}: {
	votingItems: string[];
	members: Member[];
}) {
	const totalPagesExp = '{total_pages_count_string}';

	const votingMembers = members.filter((x) => x.voting);

	const voterPlaceholders = votingMembers.map(() => '');
	const emptySlotPlaceholders = Array(emptySlotCount).fill('');
	const tableBody = votingItems.map((item) => [
		{
			content: item,
			styles: { cellWidth: 30 }
		},
		...voterPlaceholders,
		...emptySlotPlaceholders,
		{ content: '', styles: { cellWidth: 8 } },
		{ content: '', styles: { cellWidth: 8 } },
		{ content: '', styles: { cellWidth: 8 } },
		{ content: '', styles: { cellWidth: 8 } }
	]);

	const doc = new jsPDF({
		orientation: 'landscape'
	});
	// eslint-disable-next-line @typescript-eslint/no-explicit-any
	(jsPDF as any).autoTableSetDefaults({
		bodyStyles: {
			lineColor: 0,
			lineWidth: 0.05
		},
		styles: {
			font: 'NotoSansLight'
		},
		columnStyles: {
			lineColor: [255, 255, 255],
			lineWidth: 1
		},
		headStyles: {
			font: 'NotoSansBold',
			fontSize: 5,
			textColor: 0,
			lineColor: 0,
			lineWidth: 0.05,
			fillColor: '#ddd'
		}
	});

	doc.addFont('/Noto_Sans/NotoSans-Light.ttf', 'NotoSansLight', 'normal');
	doc.addFont('/Noto_Sans/NotoSans-Regular.ttf', 'NotoSansRegular', 'normal');
	doc.addFont('/Noto_Sans/NotoSans-Bold.ttf', 'NotoSansBold', 'normal');

	doc.setFont('NotoSansRegular', 'normal'); // set font
	printHeader(doc);
	doc.setFontSize(12);
	doc.text(
		'Karta do podliczania głosów',
		doc.internal.pageSize.getWidth() * 0.5,
		doc.internal.pageSize.getHeight() * 0.08,
		{
			align: 'center'
		}
	);
	doc.setFontSize(9);
	doc.text(
		'Instrukcja: Dla każdej karty do głosowania, przepisz głosy na wszystkich kandydatów, wpisując w kolumnie odpowiednio:\n' +
			'Z (Za), P (Przeciw), W (Wstrzymuje się), X (Nie głosował lub głos nieważny).\n' +
			'Po przepisaniu wszystkich głosów, zsumuj głosy Z/P/W/X na danego kandydata do odpowiednich kolumn z sumami.\n' +
			`Wygenerowano dla ${votingMembers.length} kart do głosowania (+${emptySlotCount} dodatkowych pustych miejsc.)`,
		15,
		doc.internal.pageSize.getHeight() * 0.105,
		{
			align: 'left'
		}
	);

	printPlaceDate(doc);

	doc.setFont('NotoSansLight', 'normal'); // set font
	autoTable(doc, {
		startY: doc.internal.pageSize.getHeight() * 0.22 - 15 / 2,
		pageBreak: 'auto',
		body: tableBody,
		head: [
			[
				'Kandydat / Sprawa',
				...voterPlaceholders.map((_, i) => String(i + 1)),
				...emptySlotPlaceholders.map((_, i) => `(${i + 1})`),
				'ZA',
				'PRZECIW',
				'WSTRZYMUJE SIĘ',
				'NIEWAŻNY'
			]
		],
		didDrawPage() {
			const str = `Strona ${doc.getNumberOfPages()} z ${totalPagesExp}`;

			const pageSize = doc.internal.pageSize;
			const pageHeight = pageSize.getHeight();
			doc.setFont('NotoSansRegular', 'normal');
			doc.setFontSize(10);
			doc.text(str, 5, pageHeight - 5);
		}
	});

	doc.setFont('NotoSansRegular', 'normal'); // set font

	doc.setFontSize(12);
	doc.putTotalPages(totalPagesExp);

	doc.save('counting-list.pdf');
}
